@@ -12,6 +12,7 @@ import { che as chetypes } from '@eclipse-che/api'
 import { CoreV1Api, V1Pod, Watch } from '@kubernetes/client-node'
 import axios, { AxiosInstance } from 'axios'
 import * as cp from 'child_process'
+import { cli } from 'cli-ux'
 import * as commandExists from 'command-exists'
 import * as fs from 'fs-extra'
 import * as https from 'https'
@@ -25,6 +26,7 @@ import { CHE_ROOT_CA_SECRET_NAME, DEFAULT_CA_CERT_FILE_NAME } from '../constants
 import { base64Decode } from '../util'
 
 import { CheApiClient } from './che-api-client'
+import { ChectlContext } from './context'
 import { Devfile } from './devfile'
 import { KubeHelper } from './kube'
 
@@ -100,11 +102,12 @@ export class CheHelper {
   }
 
   async cheURL(namespace = ''): Promise<string> {
-    if (!await this.cheNamespaceExist(namespace)) {
+    if (!await this.kube.getNamespace(namespace)) {
       throw new Error(`ERR_NAMESPACE_NO_EXIST - No namespace ${namespace} is found`)
     }
 
-    if (await this.kube.isOpenShift()) {
+    const ctx = ChectlContext.get()
+    if (ctx.isOpenShift) {
       return this.cheOpenShiftURL(namespace)
     } else {
       return this.cheK8sURL(namespace)
@@ -117,12 +120,13 @@ export class CheHelper {
       return this.flags['plugin-registry-url']
     }
     // check
-    if (!await this.cheNamespaceExist(namespace)) {
+    if (!await this.kube.getNamespace(namespace)) {
       throw new Error(`ERR_NAMESPACE_NO_EXIST - No namespace ${namespace} is found`)
     }
 
     // grab URL
-    if (await this.kube.isOpenShift()) {
+    const ctx = ChectlContext.get()
+    if (ctx.isOpenShift) {
       return this.chePluginRegistryOpenShiftURL(namespace)
     } else {
       return this.chePluginRegistryK8sURL(namespace)
@@ -180,22 +184,29 @@ export class CheHelper {
     throw new Error(`Secret "${CHE_ROOT_CA_SECRET_NAME}" has invalid format: "ca.crt" key not found in data.`)
   }
 
-  async saveCheCaCert(cheCaCert: string, destinaton?: string): Promise<string> {
-    if (destinaton && fs.existsSync(destinaton)) {
-      if (fs.lstatSync(destinaton).isDirectory()) {
-        destinaton = path.join(destinaton, DEFAULT_CA_CERT_FILE_NAME)
-      }
-    } else {
-      // Fallback to default location
-      destinaton = path.join(os.homedir(), DEFAULT_CA_CERT_FILE_NAME)
-    }
-
-    fs.writeFileSync(destinaton, cheCaCert)
-    return destinaton
+  async saveCheCaCert(cheCaCert: string, destination?: string): Promise<string> {
+    const cheCaCertFile = this.getTargetFile(destination)
+    fs.writeFileSync(cheCaCertFile, cheCaCert)
+    return cheCaCertFile
   }
 
   /**
-   * Retreives Keycloak admin user credentials.
+   * Handles certificate target location and returns string which points to the target file.
+   */
+  private getTargetFile(destination: string | undefined): string {
+    if (!destination) {
+      return path.join(os.tmpdir(), DEFAULT_CA_CERT_FILE_NAME)
+    }
+
+    if (fs.existsSync(destination)) {
+      return fs.lstatSync(destination).isDirectory() ? path.join(destination, DEFAULT_CA_CERT_FILE_NAME) : destination
+    }
+
+    throw new Error(`Given path \'${destination}\' doesn't exist.`)
+  }
+
+  /**
+   * Retrieves Keycloak admin user credentials.
    * Works only with installers which use Che CR (operator, olm).
    * Returns credentials as an array of two values: [login, password]
    * In case of an error an array with undefined values will be returned.
@@ -205,9 +216,10 @@ export class CheHelper {
     let adminPassword
 
     const cheCluster = await this.kube.getCheCluster(cheNamespace)
-    if (!cheCluster) {
+    if (!cheCluster || cheCluster.spec.auth.externalIdentityProvider) {
       return []
     }
+
     const keycloakCredentialsSecretName = cheCluster.spec.auth.identityProviderSecret
     if (keycloakCredentialsSecretName) {
       // Keycloak credentials are stored in secret
@@ -267,10 +279,6 @@ export class CheHelper {
     throw new Error(`ERR_ROUTE_NO_EXIST - No route ${route_names} in namespace ${namespace}`)
   }
 
-  async cheNamespaceExist(namespace = '') {
-    return this.kube.namespaceExist(namespace)
-  }
-
   async createWorkspaceFromDevfile(cheApiEndpoint: string, devfilePath: string, workspaceName?: string, accessToken?: string): Promise<chetypes.workspace.Workspace> {
     let devfile: string | undefined
     try {
@@ -410,6 +418,22 @@ export class CheHelper {
       () => { })
   }
 
+  /**
+   * Wait until workspace is in 'Active` state.
+   */
+  async waitNamespaceActive(namespaceName: string, intervalMs = 500, timeoutMs = 60000) {
+    const iterations = timeoutMs / intervalMs
+    for (let index = 0; index < iterations; index++) {
+      const namespace = await this.kube.getNamespace(namespaceName)
+      if (namespace && namespace.status && namespace.status.phase && namespace.status.phase === 'Active') {
+        return
+      }
+      await cli.wait(intervalMs)
+    }
+
+    throw new Error(`ERR_TIMEOUT: ${namespaceName} is not 'Active'.`)
+  }
+
   /**
    * Indicates if pod matches given labels.
    */
