@@ -8,10 +8,13 @@
  * SPDX-License-Identifier: EPL-2.0
  **********************************************************************/
 
-export const DEFAULT_CHE_IMAGE = 'registry.redhat.io/codeready-workspaces/server-rhel8:2.5-10'
-export const DEFAULT_CHE_OPERATOR_IMAGE = 'registry.redhat.io/codeready-workspaces/crw-2-rhel8-operator:2.5-10'
-export const DEFAULT_DEV_WORKSPACE_CONTROLLER_IMAGE = 'quay.io/devfile/devworkspace-controller:sha-4ea0394'
+// labels
+export const CHE_RELATED_COMPONENT_LABEL = 'client/org.eclipse.che=true'
 
+ // images
+export const DEFAULT_CHE_IMAGE = 'registry.redhat.io/codeready-workspaces/server-rhel8:2.6-4'
+export const DEFAULT_CHE_OPERATOR_IMAGE = 'registry.redhat.io/codeready-workspaces/crw-2-rhel8-operator:2.6-7'
+export const DEFAULT_DEV_WORKSPACE_CONTROLLER_IMAGE = 'quay.io/devfile/devworkspace-controller:sha-b434f12'
 // This image should be updated manually when needed.
 // Repository location: https://github.com/che-dockerfiles/che-cert-manager-ca-cert-generator-image
 export const CA_CERT_GENERATION_JOB_IMAGE = 'quay.io/eclipse/che-cert-manager-ca-cert-generator:671342c'
@@ -23,6 +26,12 @@ export const DEFAULT_CA_CERT_FILE_NAME = 'cheCA.crt'
 export const CHE_CLUSTER_CR_NAME = 'codeready-workspaces'
 export const CHE_CLUSTER_CRD = 'checlusters.org.eclipse.che'
 
+// operator
+export const OPERATOR_DEPLOYMENT_NAME = 'codeready-operator'
+export const CHE_OPERATOR_SELECTOR = 'app=codeready-operator'
+export const DEFAULT_CHE_NAMESPACE = 'openshift-workspaces'
+export const LEGACY_CHE_NAMESPACE = 'workspaces'
+
 // OLM
 export const DEFAULT_CHE_OLM_PACKAGE_NAME = 'codeready-workspaces'
 export const OLM_STABLE_CHANNEL_NAME = 'latest'
@@ -36,10 +45,11 @@ export const KUBERNETES_OLM_CATALOG = 'operatorhubio-catalog'
 export const OPENSHIFT_OLM_CATALOG = 'redhat-operators'
 export const CVS_PREFIX = 'crwoperator'
 export const NIGHTLY_CATALOG_SOURCE_NAME = 'eclipse-che-preview'
+export const DEFAULT_OLM_SUGGESTED_NAMESPACE = 'openshift-workspaces'
 
 // Documentation links
-export const DOC_LINK = 'https://access.redhat.com/documentation/en-us/red_hat_codeready_workspaces/2.5/'
-export const DOC_LINK_RELEASE_NOTES = 'https://access.redhat.com/documentation/en-us/red_hat_codeready_workspaces/2.5/html/release_notes_and_known_issues/index'
+export const DOC_LINK = 'https://access.redhat.com/documentation/en-us/red_hat_codeready_workspaces/2.6/'
+export const DOC_LINK_RELEASE_NOTES = 'https://access.redhat.com/documentation/en-us/red_hat_codeready_workspaces/2.6/html/release_notes_and_known_issues/index'
 export const DOCS_LINK_INSTALL_RUNNING_CHE_LOCALLY = 'https://www.eclipse.org/che/docs/che-7/overview/running-che-locally/'
 export const DOCS_LINK_IMPORT_CA_CERT_INTO_BROWSER = 'https://www.eclipse.org/che/docs/che-7/overview/importing-certificates-to-browsers/'
 export const DOCS_LINK_HOW_TO_ADD_IDENTITY_PROVIDER_OS4 = 'https://docs.openshift.com/container-platform/latest/authentication/understanding-identity-provider.html#identity-provider-overview_understanding-identity-provider'
@@ -47,5 +57,14 @@ export const DOCS_LINK_HOW_TO_CREATE_USER_OS3 = 'https://docs.openshift.com/cont
 export const DOC_LINK_OBTAIN_ACCESS_TOKEN = 'https://www.eclipse.org/che/docs/che-7/administration-guide/authenticating-users/#obtaining-the-token-from-keycloak_authenticating-to-the-che-server'
 export const DOC_LINK_OBTAIN_ACCESS_TOKEN_OAUTH = 'https://www.eclipse.org/che/docs/che-7/administration-guide/authenticating-users/#obtaining-the-token-from-openshift-token-through-keycloak_authenticating-to-the-che-server'
 
+export const OUTPUT_SEPARATOR = '-------------------------------------------------------------------------------'
+
 // DevWorkspace
 export const DEFAULT_DEV_WORKSPACE_CONTROLLER_NAMESPACE = 'devworkspace-controller'
+
+// HOOKS
+export const DEFAULT_ANALYTIC_HOOK_NAME = 'analytics'
+
+// Timeouts
+export const DEFAULT_K8S_POD_WAIT_TIMEOUT = 600000
+export const DEFAULT_K8S_POD_ERROR_RECHECK_TIMEOUT = 15000
